﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Windows;
using System.ComponentModel;

namespace BarclaysCapital.CR.PDC.Margin
{
    public abstract class ScreenControl : UserControl
    {
    }

    public abstract class BackgroundWorkerControl : ScreenControl
    {
        private BackgroundWorker _bwQuery;
        protected object _syncObject;

        protected BackgroundWorkerControl()
        {
            CreateNewBackgroundWorker();
        }

        private void CreateNewBackgroundWorker()
        {
            _bwQuery = new BackgroundWorker() { WorkerSupportsCancellation = true };
            _bwQuery.DoWork += bwQuery_DoWork;
            _bwQuery.RunWorkerCompleted += bwQuery_RunWorkerCompleted;
        }

        public void TryRunWorkerAsync(object syncObject)
        {
            // Cancel any previous background operation if not auto-refreshing and create a new one
            // The old BackgroundWorker will continue to run until finished but will be abandoned
            if (_bwQuery.IsBusy)
            {
                _bwQuery.CancelAsync();
                CreateNewBackgroundWorker();
            }

            // The member BackgroundWorker object will be a different object if cancelled above
            if (!_bwQuery.IsBusy)
            {
                _syncObject = syncObject;
                _bwQuery.RunWorkerAsync(syncObject);
            }
        }

        abstract protected internal void bwQuery_DoWork(object sender, DoWorkEventArgs e);
        abstract protected internal void bwQuery_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e);
    }
}
