﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using Infragistics.Windows.DockManager;

namespace BarclaysCapital.CR.PDC.Margin
{
    /// <summary>
    /// Interaction logic for CallAndAlertManager.xaml
    /// </summary>
    public partial class CallAndAlertManager : BackgroundWorkerControl
    {
        public CallAndAlertManager()
        {
            InitializeComponent();
        }

        public static string Header
        {
            get
            {
                return "Call & Alert Manager";
            }
        }

        public void SetData()
        {
            UpdateScreenBusyAnimation(true);
            TryRunWorkerAsync(null);
        }

        protected internal override void bwQuery_DoWork(object sender, DoWorkEventArgs e)
        {
            System.Threading.Thread.Sleep(3000);
        }

        protected internal override void bwQuery_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            UpdateScreenBusyAnimation(false);
        }

        public void UpdateScreenBusyAnimation(bool showBusy)
        {
            // Hide any visible floating panes for the duration of the busy operation as the ProcessingContentControl doesn't cover floating panes
            foreach (SplitPane splitPane in dockManager.Panes)
            {
                UpdateFloatingPanesVisibility(splitPane, showBusy ? Visibility.Collapsed : Visibility.Visible);
            }

            processingControl.IsContentProcessing = showBusy;
        }

        private void UpdateFloatingPanesVisibility(SplitPane splitPane, Visibility visibility)
        {
            foreach (FrameworkElement pane in splitPane.Panes)
            {
                if (pane is ContentPane)
                {
                    var cp = (ContentPane)pane;
                    if (cp.PaneLocation == PaneLocation.Floating)
                    {
                        var toolWindow = (PaneToolWindow)splitPane.Parent;

                        // For showing the screen as busy, only hide panes that are currently visible
                        if (visibility == Visibility.Hidden && toolWindow.Visibility == Visibility.Visible)
                        {
                            // Tag the pane to indicate its visibility must be restored at the end of the busy operation
                            toolWindow.Tag = toolWindow.Visibility;
                            toolWindow.Visibility = visibility;
                        }
                        // At the end of the busy operation, check the pane has been tagged to indicate to restore its visibility
                        else if (visibility == Visibility.Visible && toolWindow.Tag is Visibility && (Visibility)toolWindow.Tag == Visibility.Visible)
                        {
                            toolWindow.Tag = null;
                            toolWindow.Visibility = Visibility.Visible;
                        }
                        break;
                    }
                }
                else if (pane is SplitPane)
                {
                    UpdateFloatingPanesVisibility((SplitPane)pane, visibility);
                }
            }
        }
    }
}
